package ppt;
use PerlPowerTools;
# a test to get first come permissions

our $VERSION = $PerlPowerTools::VERSION;

=encoding utf8

=head1 NAME

ppt - old name for the Perl Power Tools

=head1 SYNOPSIS

See the documentation for PerlPowerTools, the new umbrella module for
this. We keep the ppt name here because it was the first name of the
project. Unfortunately, searching "ppt" is more likely to find
PowerPoint resources.

=cut

1;
